<?php
/*
 * Sign in DokuWiki via SMF database
 *
 * Features:
 *  - Full support of usernames with non-ASCII symbols
 *  - Allow to login by email (like in SMF)
 *  - Full support of user's groups
 *  - Support of getting user's list at Doku admin page with simple filter
 *  - Doesn't support any operations, which modify SMF database
 *
 * @license GPLv3 (http://www.gnu.org/licenses/gpl.html)
 * @author Vadim Nevorotin <malamut@ubuntu.ru>
 *
 * Requirements: SMF 1.x with utf8 encoding in database
 * Tested with DokuWiki 2009-12-25c "Lemming"
*/

// We need basic auth_mysql class
require_once(dirname(__FILE__).'/mysql.class.php');

/*
 * IMPORTANT! Change to your path to SMF installation!
 * Or comment two next lines and manually add all database parameters in constructor
*/
if (!defined('SMF_SETTINGS'))
	define('SMF_SETTINGS', '/home/tinycore/forum.tinycorelinux.net/Settings.php');

class auth_smf extends auth_mysql {

	/*
	 * Constructor
	*/
	function auth_smf() {
		// DokuWiki configurations array
		global $conf;
		// SMF database settings
		if (defined('SMF_SETTINGS')) {
			// If defined path to SMF settings file - include it and get all settings from this file
			include(SMF_SETTINGS);
			$this->db_server	= $db_server;
			$this->db_name		= $db_name;
			$this->db_user		= $db_user;
			$this->db_passwd	= $db_passwd;
			$this->db_prefix	= $db_prefix;
			$this->smf_url		= $boardurl;
		} else {
			// Alternativelly you can manually set all settings (don't forget to comment two lines above!)
			$this->db_server	= 'localhost';
			$this->db_name		= 'smf';
			$this->db_user		= 'smf';
			$this->db_passwd	= 'PROTECTED';
			$this->db_prefix	= 'smf_';
			$this->smf_url		= 'http://forum.domain.com';
		}
	
		// Database settings
		$conf['auth']['mysql']['server']	= $this->db_server;
		$conf['auth']['mysql']['database']	= $this->db_name;
		$conf['auth']['mysql']['user']		= $this->db_user;
		$conf['auth']['mysql']['password']	= $this->db_passwd;
		
		// Charset must be utf8, but of course you isn't so stupid to use other encoding in you database
		$conf['auth']['mysql']['charset'] = "utf8";

		// Doesn't use password encryption by DokuWiki
		$conf['auth']['mysql']['forwardClearPass'] = 1;

		/* 
		 * Minimum requirements for authorization
		*/
		// Meaning of this request is changed from auth-mysql. Look to checkPass() function below
		$conf['auth']['mysql']['checkPass']		= "SELECT member_name, passwd
													FROM {$this->db_prefix}members
													WHERE member_name = '%{user}'";
		// Ok, we can also login with email like in SMF. Look to checkPass() function below
		$conf['auth']['mysql']['checkPass-e']	= "SELECT member_name, passwd
													FROM {$this->db_prefix}members
													WHERE email_address = '%{user}'";
	 
		$conf['auth']['mysql']['getUserInfo']	= "SELECT passwd AS pass, real_name AS name, email_address AS mail
													FROM {$this->db_prefix}members
													WHERE member_name = '%{user}'";
	 
		// `group` really must be in quotes
		$conf['auth']['mysql']['getGroups']		= "SELECT '".$conf['defaultgroup']."' as `group`
													UNION
													SELECT group_name AS `group`
													FROM {$this->db_prefix}membergroups AS g, {$this->db_prefix}members AS u
													WHERE u.member_name = '%{user}'
													AND (u.id_group = g.id_group OR FIND_IN_SET(g.id_group, u.additional_groups)
														OR u.ID_POST_GROUP = g.id_group)";
		/*
		 * Minimum requirements for basic user manager support
		*/
		// We should join with group table to use filters
		$conf['auth']['mysql']['getUsers']		= "SELECT DISTINCT member_name AS user
													FROM {$this->db_prefix}members AS u
													LEFT JOIN {$this->db_prefix}membergroups AS g
													ON (u.id_group = g.id_group OR FIND_IN_SET(g.id_group, u.additional_groups)
														OR u.ID_POST_GROUP = g.id_group)";

		$conf['auth']['mysql']['FilterLogin'] = "u.member_name LIKE '%{user}'";
		$conf['auth']['mysql']['FilterName']  = "u.real_name LIKE '%{name}'";
		$conf['auth']['mysql']['FilterEmail'] = "u.email_address LIKE '%{email}'";
		$conf['auth']['mysql']['FilterGroup'] = "g.group_name LIKE '%{group}'";
		$conf['auth']['mysql']['SortOrder']   = "ORDER BY u.member_name";

		/*
		 * We don't really want to destroy our database accidentally, isn't so?
		*/
		$conf['auth']['mysql']['addUser']		= "";
		$conf['auth']['mysql']['delUser']		= "";
		$conf['auth']['mysql']['updateUser']	= "";
		$conf['auth']['mysql']['addUserGroup']	= "";
		$conf['auth']['mysql']['delUserGroup']	= "";
		$conf['auth']['mysql']['addGroup']		= "";
		$conf['auth']['mysql']['delGroup']		= "";
		$conf['auth']['mysql']['UpdateTarget']	= "";

		// Call parent constructor for other initialization
		$this->auth_mysql();
	}
	
	/*
	 * Redefine checking password function from mysql.class.php to use SMF specific password check
	 *
	 * @param  $user  user who would like access
	 * @param  $pass  user's clear text password to check
	 * @return bool
	*/
	function checkPass($user,$pass) {
		$rc  = false;

		if ($this->_openDB()) {
			$sql = str_replace('%{user}', $this->_escape($user), $this->cnf['checkPass']);
			$result = $this->_queryDB($sql);

			if ($result != false && count($result) == 1) {
				// Encrypt password like in SMF
				$encpass = sha1(strtolower($result[0]['member_name']) . $this->_escape($pass));
				// Check password...
				if ($encpass == $result[0]['passwd']) {
					$rc = true;
					// We want username to be exactly as in SMF database
					$user = $result[0]['member_name'];
				}
			} else {
				// Ok, now try to login with email
				$sql = str_replace('%{user}', $this->_escape($user), $this->cnf['checkPass-e']);
				$result = $this->_queryDB($sql);

				if($result != false && count($result) == 1) {
					// Encrypt password like in SMF
					$encpass = sha1(strtolower($result[0]['member_name']) . $this->_escape($pass));
					// Check password...
					if ($encpass == $result[0]['passwd']) {
						$rc = true;
						// We want username to be exactly as in SMF database, not email ;)
						$user = $result[0]['member_name'];
					}
				}
			}
			$this->_closeDB();
		}
		return $rc;
	}
	
}
